@@ -14,45 +14,118 @@ a valid time entry specification that mite can understand.
 - there are exactly 3 fields: project pattern, service pattern, note
 - project and service patterns match exactly one project and service each
 """
-import operator
-from dataclasses import dataclass
+from abc import ABC, abstractmethod
 from functools import singledispatch
-from typing import Any, Dict, List, Mapping, Union
+from typing import Any, Dict, List, Mapping, Sequence, Union
 
-MATCHING_PREDICATES = {"strict": operator.eq, "substring": operator.contains}
+import attrs
+import tomlkit
+from tomlkit.api import inline_table
 
 
-@dataclass
+class MatchingPredicate(ABC):
+    @abstractmethod
+    def __call__(self, entry) -> bool:
+        ...
+
+
+@attrs.frozen
 class Pattern:
+    matches: MatchingPredicate
+    definition: str
+
+    @classmethod
+    def parse(cls, pattern_data):
+        if isinstance(pattern_data, str):
+            return cls(_parse_simple(pattern_data), pattern_data)
+        definition = _as_toml(pattern_data)
+        if "project" in pattern_data:
+            if "client" in pattern_data:
+                return cls(
+                    And(
+                        [
+                            _parse_simple(pattern_data["project"]),
+                            _parse_simple(
+                                pattern_data["client"], fieldname="client_name"
+                            ),
+                        ]
+                    ),
+                    definition,
+                )
+            return cls(_parse_simple(pattern_data["project"]), definition)
+        if "client" in pattern_data:
+            raise ValueError(
+                f"Problem parsing this definition: {definition}. "
+                "Cannot filter by client only, please include a 'project' key."
+            )
+        return cls(_parse_simple(pattern_data), definition)
+
+
+def _as_toml(pattern_data):
+    tbl = inline_table()
+    for k, v in pattern_data.items():
+        if isinstance(v, str):
+            tbl[k] = v
+        else:
+            sub_tbl = inline_table()
+            sub_tbl.update(v)
+            tbl[k] = sub_tbl
+    return tomlkit.dumps(tbl)
+
+
+def _parse_simple(pattern_data, fieldname="name"):
+    if isinstance(pattern_data, str):
+        return SubstringMatch(fieldname, pattern_data)
+    if "pattern" not in pattern_data:
+        raise ValueError("'pattern' key is required.")
+    if pattern_data.get("match", "substring") == "substring":
+        return SubstringMatch(fieldname, pattern_data["pattern"])
+    return StrictMatch(fieldname, pattern_data["pattern"])
+
+
+@attrs.frozen
+class StrictMatch(MatchingPredicate):
+    fieldname: str
     pattern: str
-    match: str = "substring"
+
+    def __call__(self, entry):
+        return entry[self.fieldname] == self.pattern
 
 
-def find_unique(entries, entry_type, pattern):
+@attrs.frozen
+class SubstringMatch(MatchingPredicate):
+    fieldname: str
+    pattern: str
+
+    def __call__(self, entry):
+        return self.pattern in entry[self.fieldname]
+
+
+@attrs.frozen
+class And(MatchingPredicate):
+    predicates: Sequence[MatchingPredicate]
+
+    def __call__(self, entry) -> bool:
+        return all(pred(entry) for pred in self.predicates)
+
+
+def find_unique(entries, entry_type, pattern_data):
     """Tries to find a unique entry that matches the pattern.
 
     Ensures there is exactly one match for the given pattern.
     """
-    if isinstance(pattern, str):
-        pattern = Pattern(pattern)
-    else:
-        pattern = Pattern(**pattern)
-    pred = MATCHING_PREDICATES[pattern.match]
-    # The order matters here because operator.contains expects the filter pattern
-    # to be the second argument!
-    matches = [e for e in entries if pred(e["name"], pattern.pattern)]
-    # The validation logic is included here and not in the caller function
-    # because it is the same for services and projects.
-    if not matches:
-        raise ValueError(f"'{pattern.pattern}' did not match any {entry_type}.\n")
-    if len(matches) > 1:
+    pattern = Pattern.parse(pattern_data)
+    matched = [e for e in entries if pattern.matches(e)]
+    if not matched:
+        raise ValueError(f"'{pattern.definition}' did not match any {entry_type}.\n")
+    if len(matched) > 1:
         raise ValueError(
-            f"'{pattern.pattern}' matched the following multiple {entry_type}:\n"
-            + "\n".join(m["name"] for m in matches)
+            f"'{pattern.definition}' matched the following multiple {entry_type}:\n"
+            + "\n".join(m["name"] for m in matched)
             + "\n\n"
             + "Please provide an unambiguous pattern."
         )
-    return matches[0]
+    return matched[0]
 
 
 def to_time_entry_spec(activity, shortcuts, projects, services):
@@ -61,19 +134,30 @@ def to_time_entry_spec(activity, shortcuts, projects, services):
     if len(activity) > 3:
         raise ValueError("Activity definition too long, please enter at most 3 items.")
     values = []
-    for a in activity:
-        values += _expand(a, shortcuts)
+    for key in activity:
+        values += _expand(key, shortcuts)
+    if len(values) == 2:
+        values += [""]
     if len(values) != 3:
         raise ValueError(
-            f"Expanding your input resulted in an invalid time entry spec: {values},\n"
-            "Please check your input and shortcuts."
+            f"Cannot interpret the result of expanding your input: {values},\n"
+            "The result should have the following items (order matters!): "
+            "project, service, note (optional)."
         )
     proj_pattern, service_pattern, note = values
-    matching_project = find_unique(projects, "projects", proj_pattern)
-    matching_service = find_unique(services, "services", service_pattern)
+    matching_project = (
+        None
+        if proj_pattern == ""
+        else find_unique(projects, "projects", proj_pattern)["id"]
+    )
+    matching_service = (
+        None
+        if service_pattern == ""
+        else find_unique(services, "services", service_pattern)["id"]
+    )
     return {
-        "project_id": matching_project["id"],
-        "service_id": matching_service["id"],
+        "project_id": matching_project,
+        "service_id": matching_service,
         "note": note,
     }
 
